'use strict';

// node modules
const util = require('util');


/**
 * Module factory
 * @param {Object} rules Custom validation rules
 * @return {Validator}
 */
function ValidatorFactory (rules)
{
    return new Validator(rules);
}

/**
 * Module cosntructor
 */
function Validator (rules)
{
    /**
     * @type {Object} named regexps (default)
     */
    this.rules = {
        'uuid': /^([a-z0-9]{8}-)([a-z0-9]{4}-){3}([a-z0-9]{12})$/,
        'email': /\S+@\S+/,
        'datetime': /^\d{4}-\d{2}-\d{2}\s\d{2}:\d{2}:\d{2}(\.\d+)?$/,
        'text': /^[\S\s]+$/,
        'number': /^\d+$/,
        'bool': /^t(rue)?|f(alse)?$/i,
        'abbr': /^[A-Z]+$/,
        'alphanum': /^[a-z0-9]+$/i,
        'gender': /^[mf]$/i,
        'zip': /^[0-9\s]+$/i,
        'phone': /^\+?[0-9\s]+$/,
        'countrycode': /^[A-Z]{2}$/,
    };

    // add custom rules
    if (util.isObject(rules)) {
        for (let name in rules) {
            this.addRule(name, rules[name]);
        }
    } 
}

/**
 * @param {any} value
 * @param {String|RegExp|Function} validator
 */
Validator.prototype.validate = function (value, validator)
{
    // no validation needed
    if (empty(validator)) {
        return true;
    }
    
    let item = meta(validator);

    // skip empty value
    if (empty(value) && !item.required) {
        return true;
    }

    // invalid (empty) value
    if (empty(value) && item.required) {
        return false;
    }

    // validation
    switch (item.type) {
        case 'regexp':
            return validator.test(value);

        case 'lambda':
            return validator(value);

        default:
            let rule = this.rules[item.type];
            if (rule == undefined) {
                throw new Error(util.format('Unsupported validation type: %s', item.type));
            }
            if (util.isRegExp(rule)) {
                return rule.test(value);
            }
            if (util.isFunction(rule)) {
                return rule(value);
            }
    }
};

/**
 * @param {String} name
 * @param {RegExp|Function} validator
 * @return {Validator}
 */
Validator.prototype.addRule = function (name, validator)
{
    if (!util.isString(name)) {
        throw new Error(util.format('Parameter "name" must be string, %s given', typeof name));
    }

    if (!util.isRegExp(validator) && !util.isFunction(validator)) {
        throw new Error(util.format('Parameter "validator" must be either RegExp or Function, %s given', typeof validator));
    }

    this.rules[name] = validator;
    return this;
};

/**
 * @param {String} name
 * @return {Validator}
 */
Validator.prototype.remRule = function (name)
{
    delete this.rules[name];
    return this;    
};

/**
 * @param {String} name
 * @return {Boolean}
 */
Validator.prototype.hasRule = function (name)
{
    return !!this.rules[name];
};


module.exports = ValidatorFactory;


// Helper functions

/**
 * Parse meta info from supplied type
 * 
 * @param {String|RegExp|Function} validator
 * @returns {Object} meta info {required: bool, type: string}
 */
function meta (validator)
{
    let item = {
        type: null, 
        required: null,
    };

    // validation not needed
    if (empty(validator)) {
        return item;
    }

    // named validator
    if (util.isString(validator)) {
        let part = validator.split(':');

        item.type = part[0];
        item.required = (part[1] || 'required').match(/^null|optional$/i) 
            ? false 
            : true;
        
        return item;
    }

    // regexp validator
    if (util.isRegExp(validator)) {
        item.type = 'regexp';
        item.required = true;

        return item;
    }

    // lambda validator
    if (util.isFunction(validator)) {
        item.type = 'lambda';
        item.required = true;

        return item;
    }

    throw new Error(util.format('Unsupported validator type: %s', typeof validator));
}

/**
 * Helper method
 * 
 * @param {any} value
 * @returns {Boolean}
 */
function empty (value)
{
    if (util.isRegExp(value)) return false;
    if (util.isNullOrUndefined(value)) return true;
    if (util.isString(value) && !!value.match('^$')) return true;
    if (util.isArray(value) && !value.length) return true;
    if (util.isObject(value) && !Object.keys(value).length) return true;
    return false;
}